@@ -250,7 +250,7 @@ class S3Client {
         $chunkSize = $this->calculateOptimalChunkSize($fileSize);
         $numParts = ceil($fileSize / $chunkSize);
 
-        error_log("Starting multipart upload for {$key}: " . number_format($fileSize/1024/1024, 2) . " MB in {$numParts} parts of " . number_format($chunkSize/1024/1024, 2) . " MB each");
+        error_log("Starting multipart upload - Bucket: {$bucket}, Key: {$key}, Size: " . number_format($fileSize/1024/1024, 2) . " MB in {$numParts} parts of " . number_format($chunkSize/1024/1024, 2) . " MB each");
 
         // Check if we can resume an existing upload
         $existingUpload = null;
@@ -305,8 +305,13 @@ class S3Client {
 
                 // Read chunk from file
                 $chunkData = fread($fp, $currentPartSize);
-                if ($chunkData === false || strlen($chunkData) !== $currentPartSize) {
-                    throw new Exception("Failed to read chunk {$partNumber}");
+                if ($chunkData === false) {
+                    throw new Exception("Failed to read chunk {$partNumber}: fread returned false");
+                }
+
+                $actualSize = strlen($chunkData);
+                if ((int)$actualSize !== (int)$currentPartSize) {
+                    throw new Exception("Failed to read chunk {$partNumber}: expected {$currentPartSize} bytes, got {$actualSize} bytes (file position: " . ftell($fp) . ", file size: {$fileSize})");
                 }
 
                 // Upload part with retry logic
@@ -439,8 +444,13 @@ class S3Client {
     }
 
     private function initiateMultipartUpload($bucket, $key) {
-        $url = $this->buildUrl($bucket, $key) . '?uploads';
-        $headers = $this->buildHeaders('POST', $bucket, $key, 'uploads');
+        $url = $this->buildUrl($bucket, $key) . '?uploads=';
+        // Use empty payload hash for multipart initiate (required by AWS spec)
+        $emptyPayloadHash = 'e3b0c44298fc1c149afbf4c8996fb92427ae41e4649b934ca495991b7852b855';
+        $headers = $this->buildHeaders('POST', $bucket, $key, 'uploads=', 'application/octet-stream', $emptyPayloadHash, 0);
+
+        // Log the exact URL for debugging
+        error_log("Multipart initiate URL: {$url}");
 
         $ch = curl_init();
         curl_setopt($ch, CURLOPT_URL, $url);
@@ -452,10 +462,19 @@ class S3Client {
 
         $response = curl_exec($ch);
         $httpCode = curl_getinfo($ch, CURLINFO_HTTP_CODE);
+        $curlError = curl_error($ch);
         curl_close($ch);
 
         if ($httpCode !== 200) {
-            throw new Exception("Failed to initiate multipart upload: HTTP $httpCode");
+            $errorDetails = "HTTP $httpCode";
+            if ($curlError) {
+                $errorDetails .= ", CURL: $curlError";
+            }
+            if ($response) {
+                $errorDetails .= ", Response: " . substr($response, 0, 200);
+            }
+            error_log("Multipart upload initiation failed: {$errorDetails}");
+            throw new Exception("Failed to initiate multipart upload: {$errorDetails}");
         }
 
         // Parse upload ID from XML response
@@ -474,7 +493,10 @@ class S3Client {
         // Calculate MD5 hash for content verification
         $md5 = base64_encode(md5($data, true));
 
-        $headers = $this->buildHeaders('PUT', $bucket, $key, $queryString, 'application/octet-stream', 'UNSIGNED-PAYLOAD', strlen($data));
+        // Calculate SHA256 hash for part data (required for VersityGW)
+        $payloadHash = hash('sha256', $data);
+
+        $headers = $this->buildHeaders('PUT', $bucket, $key, $queryString, 'application/octet-stream', $payloadHash, strlen($data));
         $headers[] = 'Content-MD5: ' . $md5;
 
         $ch = curl_init();
@@ -494,16 +516,23 @@ class S3Client {
         curl_close($ch);
 
         if ($httpCode !== 200) {
+            error_log("Part upload failed for part {$partNumber}: HTTP {$httpCode}, Response: " . substr($response, 0, 500));
             throw new Exception("Failed to upload part {$partNumber}: HTTP $httpCode");
         }
 
-        // Extract ETag from response headers
-        preg_match('/ETag: "([^"]+)"/', $response, $matches);
-        if (!isset($matches[1])) {
+        // Extract ETag from response headers - try multiple formats
+        if (preg_match('/ETag:\s*"([^"]+)"/', $response, $matches)) {
+            return $matches[1];
+        } elseif (preg_match('/ETag:\s*([a-f0-9]{32})/', $response, $matches)) {
+            return $matches[1];
+        } elseif (preg_match('/etag:\s*"([^"]+)"/', $response, $matches)) {
+            return $matches[1];
+        } elseif (preg_match('/etag:\s*([a-f0-9]{32})/', $response, $matches)) {
+            return $matches[1];
+        } else {
+            error_log("Failed to extract ETag for part {$partNumber}. Response headers: " . substr($response, 0, 1000));
             throw new Exception("Failed to extract ETag for part {$partNumber}");
         }
-
-        return $matches[1];
     }
 
     private function completeMultipartUpload($bucket, $key, $uploadId, $parts) {
@@ -521,7 +550,10 @@ class S3Client {
         }
         $xml .= '</CompleteMultipartUpload>';
 
-        $headers = $this->buildHeaders('POST', $bucket, $key, $queryString, 'text/xml', 'UNSIGNED-PAYLOAD', strlen($xml));
+        // Calculate SHA256 hash of the XML payload (required for VersityGW)
+        $payloadHash = hash('sha256', $xml);
+
+        $headers = $this->buildHeaders('POST', $bucket, $key, $queryString, 'text/xml', $payloadHash, strlen($xml));
 
         $ch = curl_init();
         curl_setopt($ch, CURLOPT_URL, $url);
